//                   .-'''-.        .-'''-.                                  __                 
//         .---.   '   _    \     '   _    \ _______                   ...-'  |`. ..-'''-.     
//|        |   | /   /` '.   \  /   /` '.   \\  ___ `'.                |      |  |\.-'''\ \    
//||        |   |.   |     \  ' .   |     \  ' ' |--.\  \.-.          .-....   |  |       | |   
//||        |   ||   '      |  '|   '      |  '| |    \  '\ \        / /  -|   |  |    __/ /    
//||  __    |   |\    \     / / \    \     / / | |     |  '\ \      / /    |   |  |   |_  '.    
//||/'__ '. |   | `.   ` ..' /   `.   ` ..' /  | |     |  | \ \    / /  ...'   `--'      `.  \  
//|:/`  '. '|   |    '-...-'`       '-...-'`   | |     ' .'  \ \  / /   |         |`.      \ '. 
//||     | ||   |                              | |___.' /'    \ `  /    ` --------\ |       , | 
//||\    / '|   |                             /_______.'/      \  /      `---------'        | | 
//|/\'..' / '---'                             \_______|/       / /                         / ,' 
//'  `'-'`                                                 |`-' /                  -....--'  /  
// Version 1.1                                             '..'                   `.. __..-'  
// modding by tr13ce github lin; https://github.com/trece313
// made by DorNell8 - added a fog   
// Happy hallowen 2024

#version 120

#define BLUR_ENABLED // is blur enabled?
#define BLUR_SIZE 0.0065 // Blur size
#define NOISE_COL_SIZE 2.0 // Noise Color Size
#define DARK 3.0 // Make picture darker
#define FOG_ENABLED // is fog enabled?
#define FOG_DISTANCE 34.0 //Fog distance size [134.0 34.0 30.0 24.0]
#define FOG_COLOR vec3(0.3, 0.05, 0.1) // Dark wine red color

varying vec2 texcoord;
varying vec4 color;

uniform sampler2D colortex0;
uniform float frameTimeCounter;
uniform sampler2D depthtex0;

// Simple noise functions
float nrand(vec2 n) {
    return fract(sin(dot(n.xy, vec2(12.9898, 78.233))) * 43758.5453);
}

float n3rand(vec2 n) {
    float t = fract(frameTimeCounter);
    return (nrand(n + 0.07 * t) + nrand(n + 0.11 * t) + nrand(n + 0.13 * t)) / 3.0;
}

// Optimized Gaussian blur function
vec4 applyBlur() {
    const int mSize = 9; // Reduced size for better performance
    const int kSize = (mSize - 1) / 2;
    float kernel[mSize];
    vec3 finalColor = vec3(0.0);
    float Z = 0.0;
    float sigma = 1.0; // Adjusted for less computational cost

    // Precompute the kernel
    for (int j = 0; j <= kSize; ++j) {
        float weight = exp(-0.5 * float(j) * float(j) / (sigma * sigma)) / (sigma * sqrt(2.0 * 3.14159265359));
        kernel[kSize + j] = kernel[kSize - j] = weight;
        Z += weight * 2.0; // Accumulate weights
    }

    // Apply the blur kernel
    for (int i = -kSize; i <= kSize; ++i) {
        for (int j = -kSize; j <= kSize; ++j) {
            finalColor += kernel[kSize + i] * kernel[kSize + j] * texture(colortex0, texcoord + vec2(float(i), float(j)) * BLUR_SIZE).rgb;
        }
    }

    return vec4(finalColor / (Z * Z), 1.0);
}

void main() {
    vec3 colorSample = texture(colortex0, texcoord).rgb;

    // Calculate the average to achieve a grayscale effect
    float grayScale = dot(colorSample, vec3(0.333333)); // Average the color channels
    vec3 finalColor = vec3(grayScale); // Convert to grayscale

    #ifdef NOISE_ENABLED
    finalColor *= vec3(pow(grayScale, NOISE_COL_SIZE)) * vec3(n3rand(texcoord + vec2(float(frameTimeCounter) / 12000.0)) + 0.3);
    #endif

    vec3 bloomColor = applyBlur().rgb;
    bloomColor = vec3(pow(bloomColor.r, DARK));

    // Simplified fog calculation
    float fogDepth = texture2D(depthtex0, texcoord).r;
    #ifdef FOG_ENABLED
    float fog = clamp(pow(fogDepth / 1.001, FOG_DISTANCE) + 0.1, 0.0, 1.0);
    #else
    float fog = 0.0;
    #endif

    // Combine bloom color with fog and grayscale
    gl_FragColor = vec4(mix(bloomColor + finalColor, FOG_COLOR, fog), 1.0);
}

