#version 120

#define COLORED_SHADOWS 1
#define SHADOW_BRIGHTNESS 0.75

uniform sampler2D lightmap;
uniform sampler2D shadowcolor0;
uniform sampler2D shadowtex0;
uniform sampler2D shadowtex1;
uniform sampler2D texture;

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec4 glcolor;
varying vec4 shadowPos;

#include "/distort.glsl"

void main() {
    vec4 color = texture2D(texture, texcoord) * glcolor;
    vec2 lm = lmcoord;

    if (shadowPos.w > 0.0) { // Superficie mirando hacia la luz
        float shadowSample;
        
        #if COLORED_SHADOWS == 0
            shadowSample = texture2D(shadowtex0, shadowPos.xy).r;
        #else
            shadowSample = texture2D(shadowtex1, shadowPos.xy).r;
        #endif

        if (shadowSample < shadowPos.z) {
            // Reducir luz en las sombras
            lm.y *= SHADOW_BRIGHTNESS;
        } else {
            // Luz directa (sol)
            lm.y = mix(31.0 / 32.0 * SHADOW_BRIGHTNESS, 31.0 / 32.0, sqrt(shadowPos.w));
            
            #if COLORED_SHADOWS == 1
                // Comprobar sombras traslúcidas entre el objeto y el sol
                if (texture2D(shadowtex0, shadowPos.xy).r < shadowPos.z) {
                    vec4 shadowLightColor = texture2D(shadowcolor0, shadowPos.xy);
                    shadowLightColor.rgb = mix(vec3(1.0), shadowLightColor.rgb, shadowLightColor.a);
                    shadowLightColor.rgb = mix(shadowLightColor.rgb, vec3(1.0), lm.x);
                    color.rgb *= shadowLightColor.rgb;
                }
            #endif
        }
    }

    // Aplicar luz del mapa de luces
    color *= texture2D(lightmap, lm);

    /* DRAWBUFFERS:0 */
    gl_FragData[0] = color; // gcolor
}
